@@ -7,11 +7,12 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_COMMMANDS_LOOKUPCERTIFICATESCOMMAND_H__
-#define __KLEOPATRA_COMMMANDS_LOOKUPCERTIFICATESCOMMAND_H__
+#pragma once
 
 #include <commands/importcertificatescommand.h>
 
+#include <gpgme++/global.h>
+
 namespace Kleo
 {
 namespace Commands
@@ -26,6 +27,9 @@ public:
     explicit LookupCertificatesCommand(const QString &fingerPrint, KeyListController *parent);
     ~LookupCertificatesCommand() override;
 
+    void setProtocol(GpgME::Protocol protocol);
+    GpgME::Protocol protocol() const;
+
 private:
     void doStart() override;
     void doCancel() override;
@@ -46,4 +50,3 @@ private:
 }
 }
 
-#endif // __KLEOPATRA_COMMMANDS_LOOKUPCERTIFICATESCOMMAND_H__
