@@ -14,6 +14,8 @@
 
 #include "newcertificatewizard.h"
 
+#include <settings.h>
+
 #include "ui_chooseprotocolpage.h"
 #include "ui_enterdetailspage.h"
 #include "ui_keycreationpage.h"
@@ -21,7 +23,11 @@
 
 #include "ui_advancedsettingsdialog.h"
 
-#include "commands/exportsecretkeycommand.h"
+#ifdef QGPGME_SUPPORTS_SECRET_KEY_EXPORT
+# include "commands/exportsecretkeycommand.h"
+#else
+# include "commands/exportsecretkeycommand_old.h"
+#endif
 #include "commands/exportopenpgpcertstoservercommand.h"
 #include "commands/exportcertificatecommand.h"
 
@@ -74,6 +80,9 @@ using namespace Kleo;
 using namespace Kleo::NewCertificateUi;
 using namespace Kleo::Commands;
 using namespace GpgME;
+#ifndef QGPGME_SUPPORTS_SECRET_KEY_EXPORT
+using Kleo::Commands::Compat::ExportSecretKeyCommand;
+#endif
 
 static const char RSA_KEYSIZES_ENTRY[] = "RSAKeySizes";
 static const char DSA_KEYSIZES_ENTRY[] = "DSAKeySizes";
@@ -100,6 +109,9 @@ static const QStringList curveNames {
     { QStringLiteral("NIST P-521") },
 };
 
+namespace
+{
+
 class EmptyPassphraseProvider: public PassphraseProvider
 {
 public:
@@ -206,7 +218,7 @@ static void set_curve(QComboBox *cb, const QString &curve)
     if (!cb) {
         return;
     }
-    const int idx = cb->findText(curve);
+    const int idx = cb->findText(curve, Qt::MatchFixedString);
     if (idx < 0) {
         // Can't happen as we don't have them configurable.
         qCWarning(KLEOPATRA_LOG) << "curve " << curve << " not allowed";
@@ -263,8 +275,13 @@ static void parseAlgoString(const QString &algoString, int *size, Subkey::Pubkey
     if (lowered.startsWith(QLatin1String("ed25519"))) {
         // Special handling for this as technically
         // this is a cv25519 curve used for EDDSA
-        curve = split[0];
-        *algo = Subkey::AlgoEDDSA;
+        if (isEncrypt) {
+            curve = QLatin1String("cv25519");
+            *algo = Subkey::AlgoECDH;
+        } else {
+            curve = split[0];
+            *algo = Subkey::AlgoEDDSA;
+        }
         return;
     }
 
@@ -280,6 +297,27 @@ static void parseAlgoString(const QString &algoString, int *size, Subkey::Pubkey
     qCWarning(KLEOPATRA_LOG) << "Failed to parse default_pubkey_algo:" << algoString;
 }
 
+enum class OnUnlimitedValidity {
+    ReturnInvalidDate,
+    ReturnInternalDefault
+};
+QDate defaultExpirationDate(OnUnlimitedValidity onUnlimitedValidity)
+{
+    QDate expirationDate{};
+
+    const auto settings = Kleo::Settings{};
+    const auto defaultExpirationInDays = settings.validityPeriodInDays();
+    if (defaultExpirationInDays > 0) {
+        expirationDate = QDate::currentDate().addDays(defaultExpirationInDays);
+    } else if (defaultExpirationInDays < 0 || onUnlimitedValidity == OnUnlimitedValidity::ReturnInternalDefault) {
+        expirationDate = QDate::currentDate().addYears(2);
+    }
+
+    return expirationDate;
+}
+
+}
+
 Q_DECLARE_METATYPE(GpgME::Subkey::PubkeyAlgo)
 namespace Kleo
 {
@@ -298,30 +336,18 @@ protected:
         return static_cast<NewCertificateWizard *>(QWizardPage::wizard());
     }
 
-    QAbstractButton *button(QWizard::WizardButton button) const
+    void resetProtocol()
     {
-        return QWizardPage::wizard() ? QWizardPage::wizard()->button(button) : nullptr;
+        wizard()->resetProtocol();
     }
 
-    bool isButtonVisible(QWizard::WizardButton button) const
+    void restartAtEnterDetailsPage()
     {
-        if (const QAbstractButton *const b = this->button(button)) {
-            return b->isVisible();
-        } else {
-            return false;
-        }
+        wizard()->restartAtEnterDetailsPage();
     }
 
     QDir tmpDir() const;
 
-protected Q_SLOTS:
-    void setButtonVisible(QWizard::WizardButton button, bool visible)
-    {
-        if (QAbstractButton *const b = this->button(button)) {
-            b->setVisible(visible);
-        }
-    }
-
 protected:
 #define FIELD(type, name) type name() const { return field( QStringLiteral(#name) ).value<type>(); }
     FIELD(bool, pgp)
@@ -395,15 +421,22 @@ public:
     {
         qRegisterMetaType<Subkey::PubkeyAlgo>("Subkey::PubkeyAlgo");
         ui.setupUi(this);
-        const QDate today = QDate::currentDate();
-        ui.expiryDE->setMinimumDate(today);
-        ui.expiryDE->setDate(today.addYears(2));
-        ui.expiryCB->setChecked(true);
+        ui.expiryDE->setMinimumDate(QDate::currentDate());
         ui.emailLW->setDefaultValue(i18n("new email"));
         ui.dnsLW->setDefaultValue(i18n("new dns name"));
         ui.uriLW->setDefaultValue(i18n("new uri"));
+        ui.elgCB->setToolTip(i18nc("@info:tooltip", "This subkey is required for encryption."));
+        ui.ecdhCB->setToolTip(i18nc("@info:tooltip", "This subkey is required for encryption."));
 
         fillKeySizeComboBoxen();
+
+        connect(ui.expiryCB, &QAbstractButton::toggled,
+                this, [this](bool checked) {
+                    ui.expiryDE->setEnabled(checked);
+                    if (checked && !ui.expiryDE->isValid()) {
+                        ui.expiryDE->setDate(defaultExpirationDate(OnUnlimitedValidity::ReturnInternalDefault));
+                    }
+                });
     }
 
     void setProtocol(GpgME::Protocol proto)
@@ -412,7 +445,7 @@ public:
             return;
         }
         protocol = proto;
-        loadDefaultKeyType();
+        loadDefaults();
     }
 
     void setAdditionalUserIDs(const QStringList &items)
@@ -576,11 +609,8 @@ public:
 
     void setExpiryDate(QDate date)
     {
-        if (date.isValid()) {
-            ui.expiryDE->setDate(date);
-        } else {
-            ui.expiryCB->setChecked(false);
-        }
+        ui.expiryDE->setDate(date);
+        ui.expiryCB->setChecked(ui.expiryDE->isValid());
     }
     QDate expiryDate() const
     {
@@ -654,8 +684,11 @@ private Q_SLOTS:
 private:
     void fillKeySizeComboBoxen();
     void loadDefaultKeyType();
+    void loadDefaultExpiration();
     void loadDefaultGnuPGKeyType();
+    void loadDefaults();
     void updateWidgetVisibility();
+    void setInitialFocus();
 
 private:
     GpgME::Protocol protocol;
@@ -677,6 +710,8 @@ public:
           ui()
     {
         ui.setupUi(this);
+        ui.pgpCLB->setAccessibleDescription(ui.pgpCLB->description());
+        ui.x509CLB->setAccessibleDescription(ui.x509CLB->description());
         registerField(QStringLiteral("pgp"), ui.pgpCLB);
     }
 
@@ -734,6 +769,15 @@ public:
     {
         ui.setupUi(this);
 
+        Settings settings;
+        if (settings.hideAdvanced()) {
+            setSubTitle(i18n("Please enter your personal details below."));
+        } else {
+            setSubTitle(i18n("Please enter your personal details below. If you want more control over the parameters, click on the Advanced Settings button."));
+        }
+        ui.advancedPB->setVisible(!settings.hideAdvanced());
+        ui.resultLE->setFocusPolicy(Qt::NoFocus);
+
         // set errorLB to have a fixed height of two lines:
         ui.errorLB->setText(QStringLiteral("2<br>1"));
         ui.errorLB->setFixedHeight(ui.errorLB->minimumSizeHint().height());
@@ -773,10 +817,13 @@ public:
     bool isComplete() const override;
     void initializePage() override {
         updateForm();
+        ui.withPassCB->setVisible(pgp());
         dialog.setProtocol(pgp() ? OpenPGP : CMS);
     }
     void cleanupPage() override {
         saveValues();
+        // reset protocol when navigating back to "Choose Protocol" page
+        resetProtocol();
     }
 
 private:
@@ -794,7 +841,6 @@ private Q_SLOTS:
     void slotUpdateResultLabel()
     {
         ui.resultLE->setText(pgp() ? pgpUserID() : cmsDN());
-        ui.withPassCB->setVisible(pgp());
     }
 
 private:
@@ -870,7 +916,7 @@ private Q_SLOTS:
             setField(QStringLiteral("error"), QString());
             setField(QStringLiteral("url"), QString());
             setField(QStringLiteral("result"), i18n("Key pair created successfully.\n"
-                                                    "Fingerprint: %1", QLatin1String(result.fingerprint())));
+                                                    "Fingerprint: %1", Formatting::prettyID(result.fingerprint())));
         } else {
             QFile file(tmpDir().absoluteFilePath(QStringLiteral("request.p10")));
 
@@ -985,18 +1031,21 @@ public:
         ui.createSigningCertificatePB->setVisible(successfullyCreatedEncryptionCertificate &&!successfullyCreatedSigningCertificate);
         ui.createEncryptionCertificatePB->setVisible(successfullyCreatedSigningCertificate &&!successfullyCreatedEncryptionCertificate);
 
-        setButtonVisible(QWizard::CancelButton, error);
+        if (error) {
+            wizard()->setOptions(wizard()->options() & ~QWizard::NoCancelButtonOnLastPage);
+        } else {
+            wizard()->setOptions(wizard()->options() | QWizard::NoCancelButtonOnLastPage);
+        }
 
-        if (!initialized)
+        if (!initialized) {
             connect(ui.restartWizardPB, &QAbstractButton::clicked,
-                    wizard(), &QWizard::restart);
+                    this, [this]() {
+                        restartAtEnterDetailsPage();
+                    });
+        }
         initialized = true;
     }
 
-    void cleanupPage() override {
-        setButtonVisible(QWizard::CancelButton, true);
-    }
-
     bool isError() const
     {
         return !ui.errorTB->document()->isEmpty();
@@ -1176,11 +1225,7 @@ private:
         const bool encr = encryptionAllowed();
         setField(QStringLiteral("signingAllowed"),    !sign);
         setField(QStringLiteral("encryptionAllowed"), !encr);
-        // restart and skip to enter details Page:
-        wizard()->restart();
-        for (int i = wizard()->currentId(); i < NewCertificateWizard::EnterDetailsPageId; ++i) {
-            wizard()->next();
-        }
+        restartAtEnterDetailsPage();
     }
 
 private:
@@ -1207,6 +1252,7 @@ public:
     }
 
 private:
+    GpgME::Protocol initialProtocol = GpgME::UnknownProtocol;
     QTemporaryDir tmp;
     struct Ui {
         ChooseProtocolPage chooseProtocolPage;
@@ -1225,7 +1271,7 @@ private:
             KDAB_SET_OBJECT_NAME(keyCreationPage);
             KDAB_SET_OBJECT_NAME(resultPage);
 
-            q->setOptions(DisabledBackButtonOnLastPage);
+            q->setOptions(NoBackButtonOnStartPage|DisabledBackButtonOnLastPage);
 
             q->setPage(ChooseProtocolPageId, &chooseProtocolPage);
             q->setPage(EnterDetailsPageId,   &enterDetailsPage);
@@ -1242,13 +1288,22 @@ private:
 NewCertificateWizard::NewCertificateWizard(QWidget *p)
     : QWizard(p), d(new Private(this))
 {
-
 }
 
 NewCertificateWizard::~NewCertificateWizard() {}
 
+void NewCertificateWizard::showEvent(QShowEvent *event)
+{
+    // set WA_KeyboardFocusChange attribute to force visual focus of the
+    // focussed button when the wizard is shown (required for Breeze style
+    // and some other styles)
+    window()->setAttribute(Qt::WA_KeyboardFocusChange);
+    QWizard::showEvent(event);
+}
+
 void NewCertificateWizard::setProtocol(Protocol proto)
 {
+    d->initialProtocol = proto;
     d->ui.chooseProtocolPage.setProtocol(proto);
     setStartId(proto == UnknownProtocol ? ChooseProtocolPageId : EnterDetailsPageId);
 }
@@ -1258,6 +1313,21 @@ Protocol NewCertificateWizard::protocol() const
     return d->ui.chooseProtocolPage.protocol();
 }
 
+void NewCertificateWizard::resetProtocol()
+{
+    d->ui.chooseProtocolPage.setProtocol(d->initialProtocol);
+}
+
+void NewCertificateWizard::restartAtEnterDetailsPage()
+{
+    const auto protocol = d->ui.chooseProtocolPage.protocol();
+    restart();  // resets the protocol to the initial protocol
+    d->ui.chooseProtocolPage.setProtocol(protocol);
+    while (currentId() != NewCertificateWizard::EnterDetailsPageId) {
+        next();
+    }
+}
+
 static QString pgpLabel(const QString &attr)
 {
     if (attr == QLatin1String("NAME")) {
@@ -1274,7 +1344,7 @@ static QString attributeLabel(const QString &attr, bool pgp)
     if (attr.isEmpty()) {
         return QString();
     }
-    const QString label = pgp ? pgpLabel(attr) : Kleo::DNAttributeMapper::instance()->name2label(attr);
+    const QString label = pgp ? pgpLabel(attr) : Kleo::DN::attributeNameToLabel(attr);
     if (!label.isEmpty())
         if (pgp) {
             return label;
@@ -1408,6 +1478,7 @@ void EnterDetailsPage::updateForm()
 
     clearForm();
 
+    const auto settings = Kleo::Settings{};
     const KConfigGroup config(KSharedConfig::openConfig(), "CertificateCreationWizard");
 
     QStringList attrOrder = config.readEntry(pgp() ? "OpenPGPAttributeOrder" : "DNAttributeOrder", QStringList());
@@ -1437,6 +1508,7 @@ void EnterDetailsPage::updateForm()
         const QString label = config.readEntry(attr + QLatin1String("_label"),
                                                attributeLabel(attr, pgp()));
         const QString regex = config.readEntry(attr + QLatin1String("_regex"));
+        const QString placeholder = config.readEntry(attr + QLatin1String{"_placeholder"});
 
         int row;
         bool known = true;
@@ -1461,6 +1533,7 @@ void EnterDetailsPage::updateForm()
         }
 
         QLineEdit *le = adjust_row(ui.gridLayout, row, label, preset, validator, readonly, required);
+        le->setPlaceholderText(placeholder);
 
         const Line line = { key, label, regex, le };
         lines[row] = line;
@@ -1479,16 +1552,19 @@ void EnterDetailsPage::updateForm()
     lineList.reserve(lines.count());
     std::copy(lines.cbegin(), lines.cend(), std::back_inserter(lineList));
 
-    widgets.push_back(ui.resultLE);
+    widgets.push_back(ui.withPassCB);
     widgets.push_back(ui.advancedPB);
 
-    if (ui.nameLE->text().isEmpty()) {
+    const bool prefillName = (pgp() && settings.prefillName()) || (!pgp() && settings.prefillCN());
+    if (ui.nameLE->text().isEmpty() && prefillName) {
         ui.nameLE->setText(userFullName());
     }
-    if (ui.emailLE->text().isEmpty()) {
+    if (ui.emailLE->text().isEmpty() && settings.prefillEmail()) {
         ui.emailLE->setText(userEmailAddress());
     }
 
+    slotUpdateResultLabel();
+
     set_tab_order(widgets);
 }
 
@@ -1537,7 +1613,7 @@ static bool requirementsAreMet(const QVector<Line> &list, QString &error)
             continue;
         }
         const QString key = line.attr;
-        qCDebug(KLEOPATRA_LOG) << "requirementsAreMet(): checking \"" << key << "\" against \"" << le->text() << "\":";
+        qCDebug(KLEOPATRA_LOG) << "requirementsAreMet(): checking" << key << "against" << le->text() << ":";
         if (le->text().trimmed().isEmpty()) {
             if (key.endsWith(QLatin1Char('!'))) {
                 if (line.regex.isEmpty()) {
@@ -1773,8 +1849,8 @@ void AdvancedSettingsDialog::loadDefaultGnuPGKeyType()
         setKeyCurve(curve);
     }
 
-    if (split.size() == 2) {
-        auto algoString = split[1];
+    {
+        auto algoString = (split.size() == 2) ? split[1] : split[0];
         // If it has no usage we assume encrypt subkey
         if (!algoString.contains(QLatin1Char('/'))) {
             algoString += QStringLiteral("/enc");
@@ -1827,7 +1903,24 @@ void AdvancedSettingsDialog::loadDefaultKeyType()
     }
 
     keyTypeImmutable = config.isEntryImmutable(entry);
+}
+
+void AdvancedSettingsDialog::loadDefaultExpiration()
+{
+    if (protocol != OpenPGP) {
+        return;
+    }
+
+    setExpiryDate(defaultExpirationDate(OnUnlimitedValidity::ReturnInvalidDate));
+}
+
+void AdvancedSettingsDialog::loadDefaults()
+{
+    loadDefaultKeyType();
+    loadDefaultExpiration();
+
     updateWidgetVisibility();
+    setInitialFocus();
 }
 
 void AdvancedSettingsDialog::updateWidgetVisibility()
@@ -1905,9 +1998,57 @@ void AdvancedSettingsDialog::updateWidgetVisibility()
         ui.rsaSubCB->setChecked(false);
         ui.rsaKeyStrengthSubCB->setEnabled(false);
     }
+
     ui.expiryDE->setVisible(protocol == OpenPGP);
     ui.expiryCB->setVisible(protocol == OpenPGP);
+    const auto settings = Kleo::Settings{};
+    if (settings.isImmutable(QStringLiteral("ValidityPeriodInDays"))) {
+        ui.expiryDE->setEnabled(false);
+        ui.expiryCB->setEnabled(false);
+    }
+
     slotKeyMaterialSelectionChanged();
 }
 
+template<typename UnaryPredicate>
+bool focusFirstButtonIf(const std::vector<QAbstractButton *> &buttons, UnaryPredicate p)
+{
+    auto it = std::find_if(std::begin(buttons), std::end(buttons), p);
+    if (it != std::end(buttons)) {
+        (*it)->setFocus();
+        return true;
+    }
+    return false;
+}
+
+bool focusFirstCheckedButton(const std::vector<QAbstractButton *> &buttons)
+{
+    return focusFirstButtonIf(buttons,
+                              [](auto btn) {
+                                  return btn && btn->isEnabled() && btn->isChecked();
+                              });
+}
+
+bool focusFirstEnabledButton(const std::vector<QAbstractButton *> &buttons)
+{
+    return focusFirstButtonIf(buttons,
+                              [](auto btn) {
+                                  return btn && btn->isEnabled();
+                              });
+}
+
+void AdvancedSettingsDialog::setInitialFocus()
+{
+    // first try the key type radio buttons
+    if (focusFirstCheckedButton({ui.rsaRB, ui.dsaRB, ui.ecdsaRB})) {
+        return;
+    }
+    // then try the usage check boxes and the expiration check box
+    if (focusFirstEnabledButton({ui.signingCB, ui.certificationCB, ui.encryptionCB, ui.authenticationCB, ui.expiryCB})) {
+        return;
+    }
+    // finally, focus the OK button
+    ui.buttonBox->button(QDialogButtonBox::Ok)->setFocus();
+}
+
 #include "newcertificatewizard.moc"
